@@ -46,7 +46,7 @@ class Experiment(V1RoutesBaseModel):
     dataset_version_id: str = Field(
         description="The ID of the dataset version associated with the experiment"
     )
-    repetitions: int = Field(description="Number of times the experiment is repeated")
+    repetitions: int = Field(description="Number of times the experiment is repeated", gt=0)
     metadata: dict[str, Any] = Field(description="Metadata of the experiment")
     project_name: Optional[str] = Field(
         description="The name of the project associated with the experiment"
@@ -306,7 +306,7 @@ async def list_experiments(
                 ),
                 repetitions=experiment.repetitions,
                 metadata=experiment.metadata_,
-                project_name=None,
+                project_name=experiment.project_name,
                 created_at=experiment.created_at,
                 updated_at=experiment.updated_at,
             )
